/*
 *  OpenVPN-GUI -- A Windows GUI for OpenVPN.
 *
 *  Copyright (C) 2004 Mathias Sundman <mathias@nilings.se>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program (see the file COPYING included with this
 *  distribution); if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "options.h"
#include "main.h"
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <windows.h>


static const char usage_message[] =
  "--help\t\t\t: Show this message.\n"
  "--connect cnn \t\t: Connect to \"cnn\" at startup. (extension must be included)\n"
  "\t\t\t   Example: openvpn-gui --connect office.ovpn\n"
  "\n"
  "Options to override registry settings:\n"
  "--exe_path\t\t: Path to openvpn.exe.\n"
  "--config_dir\t\t: Path to dir to search for config files in.\n"
  "--ext_string\t\t: Extension on config files.\n"
  "--log_dir\t\t\t: Path to dir where log files will be saved.\n"
  "--priority_string\t\t: Priority string (See install.txt for more info).\n"
  "--append_string\t\t: 1=Append to log file. 0=Truncate logfile when connecting.\n"
  "--log_viewer\t\t: Path to log viewer.\n"
  "--editor\t\t\t: Path to config editor.\n"
  "--allow_edit\t\t: 1=Show Edit Config menu.\n"
  "--allow_service\t\t: 1=Show Service control menu.\n"
  "--passphrase_attempts\t: Number of passphrase attempts to allow.\n"
 ;

void
init_options (struct options *o)
{
  CLEAR (*o);

}

int Createargcargv(struct options* options, char* command_line)
{

    int    argc;
    char** argv;

    char*  arg;
    int    index;
    int    result;

    int	i;
    // count the arguments

    argc = 1;
    arg  = command_line;

    while (arg[0] != 0) {

        while (arg[0] != 0 && arg[0] == ' ') {
            arg++;
        }

        if (arg[0] != 0) {

            argc++;

            if (arg[0] == '\"') {
               arg++; 
               while (arg[0] != 0 && arg[0] != '\"') {
                  arg++;
               }
            } 
            while (arg[0] != 0 && arg[0] != ' ') {
               arg++;
            }

        }

    }    

    // tokenize the arguments
    argv = (char**)malloc(argc * sizeof(char*));

    arg = command_line;
    index = 1;

    while (arg[0] != 0) {

        while (arg[0] != 0 && arg[0] == ' ') {
            arg++;
        }

        if (arg[0] != 0) {

            if (arg[0] == '\"') {
               arg++;
               argv[index] = arg;
               index++;
               while (arg[0] != 0 && arg[0] != '\"') {
                  arg++;
               }
               if (arg[0] != 0) {
                  arg[0] = 0;    
                  arg++;
               }
               while (arg[0] != 0 && arg[0] != ' ') {
                   arg++;
               }
            }
            else {
               argv[index] = arg;
               index++;
               while (arg[0] != 0 && arg[0] != ' ') {
                   arg++;
               }
               if (arg[0] != 0) {
                   arg[0] = 0;    
                   arg++;
               }
            }


        }

    }    

    // put the program name into argv[0]
    char filename[_MAX_PATH];

    GetModuleFileName(NULL, filename, _MAX_PATH);
    argv[0] = filename;

    parse_argv(options, argc, argv);

    free(argv);
    return(0);

}

static int
add_option (struct options *options,
	    int i,
	    char *p[]);


void
parse_argv (struct options* options,
	    int argc,
	    char *argv[])
{
  int i, j;


  /* parse command line */
  for (i = 1; i < argc; ++i)
    {
      char *p[MAX_PARMS];
      CLEAR (p);
      p[0] = argv[i];
      if (strncmp(p[0], "--", 2))
	{
	  ShowMsg(GUI_NAME, "I'm trying to parse \"%s\" as an --option parameter " \
                            "but I don't see a leading '--'", p[0]);
	  exit(0);
        }
      else
	p[0] += 2;

      for (j = 1; j < MAX_PARMS; ++j)
	{
	  if (i + j < argc)
	    {
	      char *arg = argv[i + j];
	      if (strncmp (arg, "--", 2))
		p[j] = arg;
	      else
		break;
	    }
	}
      i = add_option (options, i, p);
    }
}

static int
add_option (struct options *options,
	    int i,
	    char *p[])
{

  if (streq (p[0], "help"))
    {
      MessageBox(NULL, usage_message, "OpenVPN GUI Usage", MB_OK);
      exit(0);
    }
  else if (streq (p[0], "connect") && p[1])
    {
      ++i;
      static int auto_connect_nr=0;
      if (auto_connect_nr == MAX_CONFIGS)
        {
          ShowMsg(GUI_NAME, "You can't use more than %d config-files. Please contact the author of " \
                            "OpenVPN-GUI if you have the need for more.",
                            MAX_CONFIGS);
          exit(1);
        }

      options->auto_connect[auto_connect_nr] = p[1];
      auto_connect_nr++;
    }
  else if (streq (p[0], "exe_path") && p[1])
    {
      ++i;
      strncpy(options->exe_path, p[1], sizeof(options->exe_path) - 1);
    }
  else if (streq (p[0], "config_dir") && p[1])
    {
      ++i;
      strncpy(options->config_dir, p[1], sizeof(options->config_dir) - 1);
    }
  else if (streq (p[0], "ext_string") && p[1])
    {
      ++i;
      strncpy(options->ext_string, p[1], sizeof(options->ext_string) - 1);
    }
  else if (streq (p[0], "log_dir") && p[1])
    {
      ++i;
      strncpy(options->log_dir, p[1], sizeof(options->log_dir) - 1);
    }
  else if (streq (p[0], "priority_string") && p[1])
    {
      ++i;
      strncpy(options->priority_string, p[1], sizeof(options->priority_string) - 1);
    }
  else if (streq (p[0], "append_string") && p[1])
    {
      ++i;
      strncpy(options->append_string, p[1], sizeof(options->append_string) - 1);
    }
  else if (streq (p[0], "log_viewer") && p[1])
    {
      ++i;
      strncpy(options->log_viewer, p[1], sizeof(options->log_viewer) - 1);
    }
  else if (streq (p[0], "editor") && p[1])
    {
      ++i;
      strncpy(options->editor, p[1], sizeof(options->editor) - 1);
    }
  else if (streq (p[0], "allow_edit") && p[1])
    {
      ++i;
      strncpy(options->allow_edit, p[1], sizeof(options->allow_edit) - 1);
    }
  else if (streq (p[0], "allow_service") && p[1])
    {
      ++i;
      strncpy(options->allow_service, p[1], sizeof(options->allow_service) - 1);
    }
  else if (streq (p[0], "passphrase_attempts") && p[1])
    {
      ++i;
      strncpy(options->psw_attempts_string, p[1], sizeof(options->psw_attempts_string) - 1);
    }
  else
    {
	ShowMsg(GUI_NAME, "Options error: Unrecognized option or missing parameter(s): --%s\n" \
                          "Use openvpn-gui --help for more info.", p[0]);
        exit(1);
    }
  return i;
}

LONG GetRegistryValue(HKEY regkey, char *name, char *data, DWORD len)
{
  LONG status;
  DWORD type;
  DWORD data_len;

  data_len = len;

  /* get a registry string */
  status = RegQueryValueEx(regkey, name, NULL, &type, (byte *) data, &data_len);
  if (status != ERROR_SUCCESS || type != REG_SZ)
    return(0);

  return(data_len);

}

int SetRegistryValue(HKEY regkey, char *name, char *data)
{

  /* set a registry string */
  if(RegSetValueEx(regkey, name, 0, REG_SZ, data, strlen(data)+1) != ERROR_SUCCESS)
    {
      ShowMsg(GUI_NAME, "Error writing registry value \"HKEY_CURRENT_USER\\%s\\%s\".", 
                        GUI_REGKEY_HKCU, name);
      return(0);
    }

  return(1);

}

