/*
 *  OpenVPN-GUI -- A Windows GUI for OpenVPN.
 *
 *  Copyright (C) 2004 Mathias Sundman <mathias@nilings.se>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program (see the file COPYING included with this
 *  distribution); if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#define _WIN32_IE       0x0500
#define NIF_INFO        0x00000010
// Notify Icon Infotip flags
#define NIIF_NONE       0x00000000
// icon flags are mutually exclusive
// and take only the lowest 2 bits
#define NIIF_INFO       0x00000001
#define NIIF_WARNING    0x00000002
#define NIIF_ERROR      0x00000003
#define NIIF_ICON_MASK  0x0000000F

#include <windows.h>
#include "tray.h"
#include "shellapi.h"
#include "main.h"
#include "options.h"
#include "openvpn.h"
#include "openvpn-gui-res.h"

//POPUP MENU
HMENU hMenu;
HMENU hMenuConn[MAX_CONFIGS];
HMENU hMenuService;

NOTIFYICONDATA ni;
int i;
int connected_config;

// Mouse clicks on tray
void OnNotifyTray(LPARAM lParam)
{
  POINT pt;				// point structure
  extern struct options o;
  // Right click, show the menu
  switch(lParam) {
    case WM_RBUTTONDOWN: 
      /* Re-read configs and re-create menus if no connection is running */
      if (CountConnectedState(0) == o.num_configs)
        {
          DestroyPopupMenus();
          BuildFileList();
          CreatePopupMenus();
        }

      GetCursorPos(&pt);		// get the cursors position
      SetForegroundWindow(o.hWnd);	// set the foreground window
      TrackPopupMenu(hMenu,TPM_RIGHTALIGN,pt.x,pt.y,0,o.hWnd,NULL);// track the popup
      break;

    case WM_LBUTTONDOWN:
      break;

    case WM_LBUTTONDBLCLK:
      /* Open Status window if only one connection is running */
      connected_config = -1;
      for (i=0; i < o.num_configs; i++)
        {
          if(o.cnn[i].connect_status != DISCONNECTED)
            {
              if (connected_config == -1)
                {
                  connected_config = i;
                }
              else
                {
                  connected_config = -1;
                  break;
                }
            }
        }
      if (connected_config != -1)
        {
          ShowWindow(o.cnn[connected_config].hwndStatus, SW_SHOW); 
          SetForegroundWindow(o.cnn[connected_config].hwndStatus);
        }
 
      /* Start connection if only one config exist */
      if ((o.num_configs == 1) && (o.cnn[0].connect_status == DISCONNECTED))
        StartOpenVPN(0);

      break;
  }
  PostMessage(o.hWnd,WM_NULL,0,0);// post a null message
}


void OnDestroyTray()
{
  //Destroy popup menu
  DestroyMenu(hMenu);
	
  //Remove Icon from Tray
  Shell_NotifyIcon(NIM_DELETE, &ni);
}


/* Create Popup Menus */
void CreatePopupMenus()
{
  int i;
  extern struct options o;

  /* Create Main menu */
  hMenu=CreatePopupMenu();

  /* Create a menu for every config */
  for (i=0; i < o.num_configs; i++)
    hMenuConn[i]=CreatePopupMenu();

  /* Create the Service Menu */
  hMenuService=CreatePopupMenu();

  /* Put something on the menus */
  CreateItemList(hMenu); 
} 

/* Destroy Popup Menus */
void DestroyPopupMenus()
{
  int i;
  extern struct options o;

  /* Destroy Main menu */
  DestroyMenu(hMenu);

  /* Destroy all config submenus */
  for (i=0; i < o.num_configs; i++)
    DestroyMenu(hMenuConn[i]);

  /* Destroy the Service Menu */
  DestroyMenu(hMenuService);

} 

void CreateItemList(HMENU hMenu)
{
  extern struct options o;
  extern char ext_string[];
  extern char allow_edit[2];
  extern char allow_service[2];
  int i;
 
  if (o.num_configs == 1)
    {
      /* Create Main menu with actions */
      AppendMenu(hMenu,MF_STRING, IDM_CONNECTMENU, IDM_TEXT_CONNECT);  
      AppendMenu(hMenu,MF_STRING, IDM_DISCONNECTMENU, IDM_TEXT_DISCONNECT);  
      AppendMenu(hMenu,MF_STRING, IDM_STATUSMENU, IDM_TEXT_STATUS);  
      AppendMenu(hMenu,MF_SEPARATOR,0,0);
      AppendMenu(hMenu,MF_STRING, IDM_VIEWLOGMENU, IDM_TEXT_VIEWLOG);  
      if (o.allow_edit[0]=='1')
        AppendMenu(hMenu,MF_STRING, IDM_EDITMENU, IDM_TEXT_EDITCONFIG);  
#ifndef DISABLE_CHANGE_PASSWORD
      AppendMenu(hMenu,MF_STRING, IDM_PASSPHRASEMENU, IDM_TEXT_PASSPHRASE);  
#endif

      AppendMenu(hMenu,MF_SEPARATOR,0,0);
      if (o.allow_service[0]=='1')
        {
          AppendMenu(hMenu,MF_POPUP,(UINT) hMenuService,IDM_TEXT_SERVICE);
          AppendMenu(hMenu,MF_SEPARATOR,0,0);
        }
      AppendMenu(hMenu,MF_STRING ,IDM_PROXY,IDM_TEXT_PROXY);
      AppendMenu(hMenu,MF_SEPARATOR,0,0);
      AppendMenu(hMenu,MF_STRING ,IDM_ABOUT,IDM_TEXT_ABOUT);
      AppendMenu(hMenu,MF_STRING ,IDM_CLOSE,IDM_TEXT_CLOSE);

      SetMenuStatus(0, DISCONNECTED); 

    }
  else
    {
      /* Create Main menu with all connections */
      for (i=0; i < o.num_configs; i++)
        AppendMenu(hMenu,MF_POPUP,(UINT) hMenuConn[i],o.cnn[i].config_name); 
      if (o.num_configs > 0)
        AppendMenu(hMenu,MF_SEPARATOR,0,0);
      if (o.allow_service[0]=='1')
        {
          AppendMenu(hMenu,MF_POPUP,(UINT) hMenuService,IDM_TEXT_SERVICE);
          AppendMenu(hMenu,MF_SEPARATOR,0,0);
        }
      AppendMenu(hMenu,MF_STRING ,IDM_PROXY,IDM_TEXT_PROXY);
      AppendMenu(hMenu,MF_SEPARATOR,0,0);
      AppendMenu(hMenu,MF_STRING ,IDM_ABOUT,IDM_TEXT_ABOUT);
      AppendMenu(hMenu,MF_STRING ,IDM_CLOSE,IDM_TEXT_CLOSE);
 

      /* Create PopUp menus for every connection */
      for (i=0; i < o.num_configs; i++)
        {
          AppendMenu(hMenuConn[i],MF_STRING, IDM_CONNECTMENU+i, IDM_TEXT_CONNECT);  
          AppendMenu(hMenuConn[i],MF_STRING, IDM_DISCONNECTMENU+i, IDM_TEXT_DISCONNECT);  
          AppendMenu(hMenuConn[i],MF_STRING, IDM_STATUSMENU+i, IDM_TEXT_STATUS);  
          AppendMenu(hMenuConn[i],MF_SEPARATOR,0,0);
          AppendMenu(hMenuConn[i],MF_STRING, IDM_VIEWLOGMENU+i, IDM_TEXT_VIEWLOG);  
          if (o.allow_edit[0]=='1')
            AppendMenu(hMenuConn[i],MF_STRING, IDM_EDITMENU+i, IDM_TEXT_EDITCONFIG);  
#ifndef DISABLE_CHANGE_PASSWORD
          AppendMenu(hMenuConn[i],MF_STRING, IDM_PASSPHRASEMENU+i, IDM_TEXT_PASSPHRASE);  
#endif

          SetMenuStatus(i, DISCONNECTED); 
        }
    }

  /* Create Service menu */
  AppendMenu(hMenuService,MF_STRING, IDM_SERVICE_START, IDM_TEXT_SERVICE_START);  
  AppendMenu(hMenuService,MF_STRING, IDM_SERVICE_STOP, IDM_TEXT_SERVICE_STOP);  
  AppendMenu(hMenuService,MF_STRING, IDM_SERVICE_RESTART, IDM_TEXT_SERVICE_RESTART);  

}


BOOL LoadAppIcon()
{
  extern struct options o;

  // Load icon from resource
  HICON hIcon = (HICON)LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(APP_ICON), 
                                                IMAGE_ICON, 0, 0, LR_DEFAULTCOLOR);
  if (hIcon) {
    SendMessage(o.hWnd, WM_SETICON, (WPARAM) (ICON_SMALL), (LPARAM) (hIcon));
    SendMessage(o.hWnd, WM_SETICON, (WPARAM) (ICON_BIG), (LPARAM) (hIcon));  //ALT+TAB icon
    return TRUE;
  }
  return FALSE;
}


void ShowTrayIcon()
{
  extern struct options o;

  ni.cbSize = sizeof(ni);
  ni.uID = 0;
  lstrcpyn(ni.szTip, MSG_TIP, sizeof(ni.szTip)); 
  ni.hWnd = o.hWnd;
  ni.uFlags = NIF_MESSAGE | NIF_TIP | NIF_ICON; // We want to use icon, tip, and callback message
  ni.uCallbackMessage = WM_NOTIFYICONTRAY;      // Our custom callback message (WM_APP + 1)
    
     
  //Load selected icon
  ni.hIcon = (HICON)LoadIcon((HINSTANCE)GetWindowLong(o.hWnd, GWL_HINSTANCE), 
                                                       MAKEINTRESOURCE(APP_ICON_DISCONNECTED));	

  Shell_NotifyIcon(NIM_ADD, &ni);       

}

/* SetTrayIcon(int connected)
 * connected=0 -> DisConnected
 * connected=1 -> Connecting
 * connected=2 -> Connected
 */
void SetTrayIcon(int connected)
{
  char msg[500];
  char conn_name[200];
  int i, first_conn;
  extern struct options o;

  ni.cbSize = sizeof(ni);
  ni.uID = 0;
  ni.hWnd = o.hWnd;
  ni.uFlags = NIF_MESSAGE | NIF_TIP | NIF_ICON; // We want to use icon, tip, and callback message
  ni.uCallbackMessage = WM_NOTIFYICONTRAY;      // Our custom callback message (WM_APP + 1)
    
  strncpy(msg, MSG_TIP, sizeof(ni.szTip)); 

  first_conn=1;
  for (i=0; i < o.num_configs; i++)
    {
      if(o.cnn[i].connect_status == CONNECTED)
        {
          /* Cut of extention from config filename. */
          strncpy(conn_name, o.cnn[i].config_file, sizeof(conn_name));
          conn_name[strlen(conn_name) - (strlen(o.ext_string)+1)]=0;

          /* Append connection name to Icon Tip Msg */
          if (first_conn)
            strncat(msg, "\nConnected to: ", sizeof(msg) - strlen(msg) - 1);
          else
            strncat(msg, ", ", sizeof(msg) - strlen(msg) - 1);
          strncat(msg, conn_name, sizeof(msg) - strlen(msg) - 1);
          first_conn=0;
        }
    }
  strncpy(ni.szTip, msg, sizeof(ni.szTip)); 
     
  //Load selected icon
  if (connected==2) 
    ni.hIcon = (HICON)LoadIcon((HINSTANCE)GetWindowLong(o.hWnd, GWL_HINSTANCE), 
                                                       MAKEINTRESOURCE(APP_ICON_CONNECTED));	
  else if (connected==1)
    ni.hIcon = (HICON)LoadIcon((HINSTANCE)GetWindowLong(o.hWnd, GWL_HINSTANCE), 
                                                       MAKEINTRESOURCE(APP_ICON_CONNECTING));	
  else if (connected==0)
    ni.hIcon = (HICON)LoadIcon((HINSTANCE)GetWindowLong(o.hWnd, GWL_HINSTANCE), 
                                                       MAKEINTRESOURCE(APP_ICON_DISCONNECTED));	

  Shell_NotifyIcon(NIM_MODIFY, &ni);       
}

void ShowTrayBalloon(char *infotitle_msg, char *info_msg)
{
  extern struct options o;

  ni.cbSize = sizeof(ni);
  ni.uID = 0;
  ni.hWnd = o.hWnd;
  ni.uFlags = NIF_INFO; 			/* We want to show a balloon */
  ni.uTimeout = 5000;
  ni.dwInfoFlags = NIIF_INFO;			/* Show an Info Icon */
  strncpy(ni.szInfo, info_msg, sizeof(ni.szInfo)); 
  strncpy(ni.szInfoTitle, infotitle_msg, sizeof(ni.szInfoTitle)); 

  Shell_NotifyIcon(NIM_MODIFY, &ni);       
}


void SetMenuStatus (int config, int bCheck)
{
  /* bCheVck values:
   * 0 - Not Connected
   * 1 - Connecting
   * 2 - Connected
   * 4 - Disconnecting
   */


  int iState;
  extern struct options o;

  if (o.num_configs == 1)
    {
      if (bCheck == DISCONNECTED)
        {
          EnableMenuItem(hMenu, IDM_CONNECTMENU, MF_ENABLED);
          EnableMenuItem(hMenu, IDM_DISCONNECTMENU, MF_GRAYED);
          EnableMenuItem(hMenu, IDM_STATUSMENU, MF_GRAYED);
        }
      else if (bCheck == CONNECTING)
        {
          EnableMenuItem(hMenu, IDM_CONNECTMENU, MF_GRAYED);
          EnableMenuItem(hMenu, IDM_DISCONNECTMENU, MF_ENABLED);
          EnableMenuItem(hMenu, IDM_STATUSMENU, MF_ENABLED);
        }
      else if (bCheck == CONNECTED)
        {
          EnableMenuItem(hMenu, IDM_CONNECTMENU, MF_GRAYED);
          EnableMenuItem(hMenu, IDM_DISCONNECTMENU, MF_ENABLED);
          EnableMenuItem(hMenu, IDM_STATUSMENU, MF_ENABLED);
        }
      else if (bCheck == DISCONNECTING)
        {
          EnableMenuItem(hMenu, IDM_CONNECTMENU, MF_GRAYED);
          EnableMenuItem(hMenu, IDM_DISCONNECTMENU, MF_GRAYED);
          EnableMenuItem(hMenu, IDM_STATUSMENU, MF_ENABLED);
        }
    }
  else 
    {
      iState = ((bCheck == CONNECTED) || (bCheck == DISCONNECTING)) ? 
               MF_CHECKED : MF_UNCHECKED ;
      CheckMenuItem (hMenu, (UINT) hMenuConn[config], iState) ;

      if (bCheck == DISCONNECTED)
        {
          EnableMenuItem(hMenuConn[config], IDM_CONNECTMENU + config, MF_ENABLED);
          EnableMenuItem(hMenuConn[config], IDM_DISCONNECTMENU + config, MF_GRAYED);
          EnableMenuItem(hMenuConn[config], IDM_STATUSMENU + config, MF_GRAYED);
        }
      else if (bCheck == CONNECTING)
        {
          EnableMenuItem(hMenuConn[config], IDM_CONNECTMENU + config, MF_GRAYED);
          EnableMenuItem(hMenuConn[config], IDM_DISCONNECTMENU + config, MF_ENABLED);
          EnableMenuItem(hMenuConn[config], IDM_STATUSMENU + config, MF_ENABLED);
        }
      else if (bCheck == CONNECTED)
        {
          EnableMenuItem(hMenuConn[config], IDM_CONNECTMENU + config, MF_GRAYED);
          EnableMenuItem(hMenuConn[config], IDM_DISCONNECTMENU + config, MF_ENABLED);
          EnableMenuItem(hMenuConn[config], IDM_STATUSMENU + config, MF_ENABLED);
        }
      else if (bCheck == DISCONNECTING)
        {
          EnableMenuItem(hMenuConn[config], IDM_CONNECTMENU + config, MF_GRAYED);
          EnableMenuItem(hMenuConn[config], IDM_DISCONNECTMENU + config, MF_GRAYED);
          EnableMenuItem(hMenuConn[config], IDM_STATUSMENU + config, MF_ENABLED);
        }
    }

}

void SetServiceMenu(BOOL bCheck)
{
  /* 1 = Service is started.
   * 0 = Service is not started."
   */
  if (bCheck) {
    EnableMenuItem(hMenuService, IDM_SERVICE_START, MF_GRAYED);
    EnableMenuItem(hMenuService, IDM_SERVICE_STOP, MF_ENABLED);
    EnableMenuItem(hMenuService, IDM_SERVICE_RESTART, MF_ENABLED);
  }
  else {
    EnableMenuItem(hMenuService, IDM_SERVICE_START, MF_ENABLED);
    EnableMenuItem(hMenuService, IDM_SERVICE_STOP, MF_GRAYED);
    EnableMenuItem(hMenuService, IDM_SERVICE_RESTART, MF_GRAYED);
  }
}


void SetServiceMenuDisabled()
{
    EnableMenuItem(hMenuService, IDM_SERVICE_START, MF_GRAYED);
    EnableMenuItem(hMenuService, IDM_SERVICE_STOP, MF_GRAYED);
    EnableMenuItem(hMenuService, IDM_SERVICE_RESTART, MF_GRAYED);
}


